@@ -24,7 +24,6 @@
    (declare (ignore second minute hour date month year dst-p tz))
    day-of-week))
 
-
 ;; parse the date to
 (defun date-parse(date)
   (if (= 8 (length date))
@@ -41,7 +40,7 @@
          :year year))
     nil))
 
-(defun post(&optional &key title tag date id (tiny nil) (author nil) (converter nil))
+(defun post(&optional &key title tag date id (tiny nil) (author (getf *config* :webmaster)) (converter nil))
   (push (make-article :title title
                       :tag tag
                       :date (date-parse date)
@@ -101,14 +100,10 @@
 ;; we escape ~ to avoid failures with format
 (defun load-file(path)
   (if (probe-file path)
-      (replace-all
-       (apply #'concatenate 'string
-              (with-open-file (stream path)
-                (loop for line = (read-line stream nil)
-                   while line
-                   collect
-                   (format nil "~a~%" line))))
-       "~" "~~")
+      (with-open-file (stream path)
+        (let ((contents (make-string (file-length stream))))
+          (read-sequence contents stream)
+          contents))
     (progn
       (format t "ERROR : file ~a not found. Aborting~%" path)
       (quit))))
@@ -116,7 +111,7 @@
 ;; save a string in a file
 (defun save-file(path data)
   (with-open-file (stream path :direction :output :if-exists :supersede)
-		  (format stream data)))
+		  (write-sequence data stream)))
 
 ;; simplify the str replace work
 (defmacro template(before &body after)
@@ -134,24 +129,27 @@
 
 ;; generate the html file from the source file
 ;; using the converter associated with the post
-(defun use-converter-to-html(article)
-  (with-converter
-   (let ((output (converter-command converter-object)))
-     (let* ((src-file (format nil "~a~a" (article-id article) (converter-extension converter-object)))
-	   (dst-file (format nil "temp/data/~a.html" (article-id article) ))
-	   (full-src-file (format nil "data/~a" src-file)))
-       ;; skip generating if the destination exists
-       ;; and is more recent than source
-       (unless (and
-		(probe-file dst-file)
-		(>=
-		 (file-write-date dst-file)
-		 (file-write-date full-src-file)))
-	 (ensure-directories-exist "temp/data/")
-	 (template "%IN" src-file)
-	 (template "%OUT" dst-file)
-	 (format t "~a~%" output)
-	 (uiop:run-program output))))))
+(defun use-converter-to-html(filename &optional (converter-name nil))
+  (let* ((converter-object (getf *converters*
+                                 (or converter-name
+			             converter-name
+			             (getf *config* :default-converter))))
+         (output           (converter-command converter-object))
+         (src-file (format nil "~a~a" filename (converter-extension converter-object)))
+         (dst-file (format nil "temp/data/~a.html" filename ))
+         (full-src-file (format nil "data/~a" src-file)))
+      ;; skip generating if the destination exists
+      ;; and is more recent than source
+      (unless (and
+               (probe-file dst-file)
+               (>=
+                (file-write-date dst-file)
+                (file-write-date full-src-file)))
+        (ensure-directories-exist "temp/data/")
+        (template "%IN" src-file)
+        (template "%OUT" dst-file)
+        (format t "~a~%" output)
+        (uiop:run-program output))))
 
 ;; format the date
 (defun date-format(format date)
@@ -175,6 +173,22 @@
   `(progn
      (save-file ,name (generate-layout ,@data))))
 
+;; generate a gemini index file
+(defun generate-gemini-index(articles)
+  (let ((output (load-file "templates/gemini_head.tpl")))
+    (dolist (article articles)
+      (setf output
+	    (string
+	     (concatenate 'string output
+                          (format nil "=> ~a/articles/~a.md ~a-~2,'0d-~2,'0d ~a~%"
+                                  (getf *config* :gemini-path)
+                                  (article-id article)
+                                  (getf (article-date article) :year)
+                                  (getf (article-date article) :monthnumber)
+                                  (getf (article-date article) :daynumber)
+                                  (article-title article))))))
+    output))
+
 ;; generate a gopher index file
 (defun generate-gopher-index(articles)
   (let ((output (load-file "templates/gopher_head.tpl")))
@@ -245,7 +259,7 @@
 	   (template "%%Text%%"   (if no-text
 				      ""
                                       (if (and tiny (article-tiny article))
-                                          (article-tiny article)
+                                          (format nil "<p>~a</p>" (article-tiny article))
                                           (load-file (format nil "temp/data/~d.html" (article-id article))))))))
 
 ;; return a html string
@@ -312,9 +326,9 @@
 
   ;; produce each article file
   (loop for article in *articles*
-	do
-	;; use the article's converter to get html code of it
-	(use-converter-to-html article)
+     do
+     ;; use the article's converter to get html code of it
+       (use-converter-to-html (article-id article) (article-converter article))
 
 	(generate  (format nil "output/html/~d-~d.html"
 			   (date-format "%Year-%MonthNumber-%DayNumber"
@@ -341,6 +355,59 @@
   ;;(generate-file-rss)
   (save-file "output/html/rss.xml" (generate-rss)))
 
+;; we do all the gemini capsule
+(defun create-gemini-capsule()
+
+  ;; produce the index.md file
+  (save-file (concatenate 'string "output/gemini/" (getf *config* :gemini-index))
+             (generate-gemini-index *articles*))
+
+  ;; produce a tag list menu
+  (let* ((directory-path "output/gemini/_tags_/")
+         (index-path (concatenate 'string directory-path (getf *config* :gemini-index))))
+    (ensure-directories-exist directory-path)
+    (save-file index-path
+               (let ((output (load-file "templates/gemini_head.tpl")))
+                 (loop for tag in
+                      ;; sort tags per articles in it
+                      (sort (articles-by-tag) #'>
+                            :key #'(lambda (x) (length (getf x :value))))
+                    do
+                      (setf output
+	                    (string
+	                     (concatenate
+                              'string output
+                              (format nil "=> ~a/~a/index.md ~a ~d~%"
+                                      (getf *config* :gemini-path)
+                                      (getf tag :name)
+                                      (getf tag :name)
+                                      (length (getf tag :value)))))))
+                 output)))
+
+  ;; produce each tag gemini index
+  (loop for tag in (articles-by-tag) do
+       (let* ((directory-path (concatenate 'string "output/gemini/" (getf tag :NAME) "/"))
+              (index-path (concatenate 'string directory-path (getf *config* :gemini-index)))
+              (articles-with-tag (loop for article in *articles*
+                                    when (member (article-id article) (getf tag :VALUE) :test #'equal)
+                                    collect article)))
+         (ensure-directories-exist directory-path)
+         (save-file index-path (generate-gemini-index articles-with-tag))))
+
+  ;; produce each article file (adding some headers)
+  (loop for article in *articles*
+     do
+       (with-converter
+	   (let ((id (article-id article)))
+	     (save-file (format nil "output/gemini/articles/~a.txt" id)
+                        (format nil "~{~a~}"
+                                (list
+                                 "Title : " (article-title article) #\Newline
+                                 "Author: " (article-author article) #\Newline
+                                 "Date  : " (date-format (getf *config* :date-format) (article-date article)) #\Newline
+                                 "Tags  : " (article-tag article) #\Newline #\Newline
+		                 (load-file (format nil "data/~d~d" id (converter-extension converter-object))))))))))
+
 ;; we do all the gopher hole
 (defun create-gopher-hole()
 
@@ -394,19 +461,26 @@
          (ensure-directories-exist directory-path)
          (save-file index-path (generate-gopher-index articles-with-tag))))
 
-  ;; produce each article file (only a copy/paste in fact)
+  ;; produce each article file (adding some headers)
   (loop for article in *articles*
-	do
-	(with-converter
-	 (let ((id (article-id article)))
-	   (save-file (format nil "output/gopher/article-~d.txt" id)
-		      (load-file (format nil "data/~d~d" id (converter-extension converter-object))))))))
+     do
+       (with-converter
+	   (let ((id (article-id article)))
+	     (save-file (format nil "output/gopher/article-~d.txt" id)
+                        (format nil "Title: ~a~%Author: ~a~%Date: ~a~%Tags: ~a~%============~%~%~a"
+                                 (article-title article)
+                                 (article-author article)
+                                 (date-format (getf *config* :date-format) (article-date article))
+                                 (article-tag article)
+		                         (load-file (format nil "data/~d~d" id (converter-extension converter-object)))))))))
 
 
 ;; This is function called when running the tool
 (defun generate-site()
   (if (getf *config* :html)
       (create-html-site))
+  (if (getf *config* :gemini)
+      (create-gemini-capsule))
   (if (getf *config* :gopher)
       (create-gopher-hole)))
 
